from collections.abc import Callable

from jbqt.common.consts import GlobalRefs


def debug_scroll_pos(func: Callable):
    def wrapper(self, *args, **kwargs):
        if GlobalRefs.scroll_area is None:
            return func(self, *args, **kwargs)
        focus = None
        if GlobalRefs.main_window:
            focus = GlobalRefs.main_window.focusWidget()
        print(f"scroll pos {func.__name__}")
        print("before", GlobalRefs.scroll_area.verticalScrollBar().value(), focus)
        result = func(self, *args, **kwargs)

        if GlobalRefs.main_window:
            focus = GlobalRefs.main_window.focusWidget()
        print(
            f"after {GlobalRefs.scroll_area.verticalScrollBar().value()} {focus}\n"
        )

        return result

    return wrapper


def debug_scroll_pos_no_args(func: Callable):
    def wrapper(self):
        if GlobalRefs.scroll_area is None:
            return func(self)
        focus = None
        if GlobalRefs.main_window:
            focus = GlobalRefs.main_window.focusWidget()
        print(f"scroll pos {func.__name__}")
        print("before", GlobalRefs.scroll_area.verticalScrollBar().value(), focus)
        result = func(self)
        if GlobalRefs.main_window:
            focus = GlobalRefs.main_window.focusWidget()
        print(
            f"after {GlobalRefs.scroll_area.verticalScrollBar().value()} {focus}\n"
        )

        return result

    return wrapper


def _call_func(func: Callable, *args, **kwargs):
    if args and kwargs:
        return func(*args, **kwargs)
    elif args:
        return func(*args)
    elif kwargs:
        return func(**kwargs)
    else:
        return func()


def preserve_scroll(func: Callable):
    def wrapper(self, *args, **kwargs):
        if GlobalRefs.scroll_area is None:
            return _call_func(func, self, *args, **kwargs)

        scroll_pos = GlobalRefs.scroll_area.verticalScrollBar().value()
        focus = GlobalRefs.app.focusWidget()
        result = _call_func(
            func,
            self,
            *args,
            **kwargs,
        )
        GlobalRefs.scroll_area.verticalScrollBar().setValue(scroll_pos)
        focus.setFocus()
        return result

    return wrapper



from typing import Any, Callable

from PyQt6.QtCore import pyqtSignal
from PyQt6.QtWidgets import (
    QDialog,
    QDialogButtonBox,
    QDoubleSpinBox,
    QLabel,
    QLineEdit,
    QSpinBox,
    QVBoxLayout,
)


INPUT_SIGNAL_TYPES: tuple[Any] = (str, int, float)


class InputDialog(QDialog):
    signal_types: tuple[Any] = INPUT_SIGNAL_TYPES
    """ reference value for external use """

    value = pyqtSignal(*([ptype] for ptype in INPUT_SIGNAL_TYPES))

    def __init__(
        self,
        parent=None,
        title: str = "Input",
        msg_str: str = "Enter Input:",
        input_type: type = str,
        **opts,
    ):
        super().__init__(parent)

        self.setWindowTitle(title)

        QBtn = (
            QDialogButtonBox.StandardButton.Cancel
            | QDialogButtonBox.StandardButton.Open
        )

        self.buttonBox = QDialogButtonBox(QBtn)
        self.buttonBox.accepted.connect(self.accept)
        self.buttonBox.rejected.connect(self.reject)

        layout = QVBoxLayout()
        message = QLabel(msg_str)
        layout.addWidget(message)

        self.input_widget = None
        if input_type is str:
            self.input_widget = QLineEdit()
        elif input_type is int:
            self.input_widget = QSpinBox()
        elif input_type is float:
            self.input_widget = QDoubleSpinBox()

        self._init_widget(**opts)

        layout.addWidget(self.input_widget)
        layout.addWidget(self.buttonBox)
        self.setLayout(layout)

    def _init_widget(self, **opts) -> None:
        for key, value in opts.items():
            setter_key = f"set{key[0].upper() + key[1:]}"
            if hasattr(self.input_widget, setter_key):
                setter = getattr(self.input_widget, setter_key)
                if setter:
                    setter(value)

    def connect(self, callback: Callable) -> None:
        for parm_type in self.signal_types:
            self.value[parm_type].connect(callback)

    def accept(self):
        # Emit the custom signal with data when the dialog is accepted

        if isinstance(self.input_widget, QLineEdit):
            self.value[str].emit(self.input_widget.text())
        elif isinstance(self.input_widget, QSpinBox):
            self.value[int].emit(self.input_widget.value())
        elif isinstance(self.input_widget, QDoubleSpinBox):
            self.value[float].emit(self.input_widget.value())

        super().accept()



"""Model for chips widget"""

from abc import abstractmethod, ABC

from PyQt6.QtCore import QObject
from PyQt6.QtWidgets import QWidget


class IChipButton(QWidget):
    """ChipButton model"""

    def __init__(self, parent: QObject = None) -> None:
        super().__init__(parent)



"""Common exports for jbqt"""

from jbqt.common import consts, qt_utils
from jbqt.common.consts import GlobalRefs
from jbqt.common.qt_utils import register_app

__all__ = [
    "GlobalRefs",
    "consts",
    "qt_utils",
    "register_app",
]



"""Const exports"""

from jbqt.common import consts, qt_utils
from jbqt.common.consts import COLORS, STYLES, SIZES, GlobalRefs, ICONS
from jbqt.common.qt_utils import register_app

__all__ = [
    "COLORS",
    "consts",
    "GlobalRefs",
    "ICONS",
    "qt_utils",
    "register_app",
    "SIZES",
    "STYLES",
]



"""Model for chips widget"""

from abc import abstractmethod, ABC

from PyQt6.QtCore import QObject
from PyQt6.QtWidgets import QWidget

from jbqt.models.chip_button import IChipButton


class IChipsWidget(QWidget):
    """ChipsWidget model"""

    def __init__(self, parent: QObject = None) -> None:
        super().__init__(parent)
        self.values: list = None

    def add_chip(self):
        pass

    def remove_chip(self, button: IChipButton):
        pass

    def add_chips(self, items: list[str]) -> None:
        pass

    def remove_chips(self, items: list[str]) -> None:
        pass

    def remove_all(self, *_) -> None:
        pass



from PyQt6.QtCore import Qt, QTimer, QRect
from PyQt6.QtWidgets import QLabel


class Toast(QLabel):
    def __init__(self, message, duration=2000):
        super().__init__(message)

        # Set the basic appearance
        self.setWindowFlags(
            Qt.WindowType.FramelessWindowHint | Qt.WindowType.ToolTip
        )
        self.setAttribute(Qt.WidgetAttribute.WA_TransparentForMouseEvents, True)
        self.setStyleSheet(
            "background-color: black; color: white; padding: 10px; border-radius: 5px;"
        )
        self.setAlignment(Qt.AlignmentFlag.AlignCenter)

        # Set the duration for the toast
        self.timer = QTimer(self)
        self.timer.timeout.connect(self.hide)
        self.timer.setInterval(duration)

    def show_toast(self, parent):
        self.setParent(parent)
        self.adjustSize()

        # Center the toast in the parent widget
        parent_rect: QRect = parent.rect()
        move_point = parent_rect.center() - self.rect().center()
        move_point.setY(20)
        self.move(move_point)
        # self.move((parent_rect.center() / 2) - self.rect().center())
        self.show()
        self.timer.start()



from re import Pattern

from PyQt6.QtCore import QObject, pyqtSignal, Qt
from PyQt6.QtGui import QCursor, QMouseEvent, QWheelEvent
from PyQt6.QtWidgets import QComboBox, QListWidget, QListWidgetItem

import jbutils as utils
from jbqt.common import consts, qt_utils


class MultiSelectComboBox(QComboBox):
    selectedChanged = pyqtSignal(list)

    def __init__(
        self,
        data: list | dict = None,
        selected: list[str] = None,
        multi_enabled: bool = True,
        use_weight: bool = False,
        weight_re: Pattern = None,
    ):
        super().__init__()

        self.use_weight = use_weight
        if use_weight:
            weight_re = weight_re or consts.TAG_RE

        self.weight_re = weight_re

        self._multi_select_enabled: bool = multi_enabled
        # Make the combo box read-only and use a custom view
        self.setEditable(True)
        self.lineEdit().setReadOnly(True)
        # Override the mouse press event of the QLineEdit
        self.lineEdit().installEventFilter(self)

        self.view: QListWidget = QListWidget()
        self._selected = selected or []

        # Set the custom view
        self.setModel(self.view.model())
        self.setView(self.view)

        # Add items and categories
        if isinstance(data, dict):
            for name, items in data.items():
                self.add_group(name, items)
        elif isinstance(data, list):
            self.add_items(data)

        # Connect the item clicked signal
        self.view.itemClicked.connect(self.toggle_check_state)
        self.update_display_text()

    def eventFilter(self, obj: QObject, event: QMouseEvent) -> bool:

        if (
            obj == self.lineEdit()
            and event.type() == QMouseEvent.Type.MouseButtonRelease
        ):
            self.showPopup()

            return True
        return super().eventFilter(obj, event)

    def wheelEvent(self, event: QWheelEvent):
        # Check if the mouse cursor is within the bounds of the line edit
        pos = self.mapFromGlobal(QCursor.pos())
        if self.lineEdit().rect().contains(pos):
            # Consume the wheel event to prevent scrolling
            return

        # If not over the line edit, let the base class handle it (scrolling)
        super().wheelEvent(event)

    def toggle_check_state(self, item: QListWidgetItem):
        # Toggle check state if the item is selectable
        if item.flags() & Qt.ItemFlag.ItemIsUserCheckable:
            current_state = item.checkState()

            new_state = (
                Qt.CheckState.Unchecked
                if current_state == Qt.CheckState.Checked
                else Qt.CheckState.Checked
            )
            item.setCheckState(new_state)
            if not self._multi_select_enabled:
                self.toggle_others(item)

            self.update_display_text()
            self._selected_changed()

    def toggle_others(self, selected: QListWidgetItem) -> None:

        for i in range(self.view.count()):
            item = self.view.item(i)
            if item != selected:
                item.setCheckState(Qt.CheckState.Unchecked)
        self.update_display_text()

    def setMultiSelectEnabled(self, value: bool) -> None:
        self._multi_select_enabled = value

    def multiSelectEnabled(self) -> bool:
        return self._multi_select_enabled

    def add_items(self, items: list[str] | dict, indent: int = 0) -> None:

        indent_str = "    " * indent
        if isinstance(items, dict):
            for name, children in items.items():
                self.add_group(name, children, indent + 1)
        else:
            for item_value in items:
                if isinstance(item_value, tuple):
                    item = QListWidgetItem(f"{indent_str}{item_value[0]}")
                    item.setData(consts.LIST_ITEM_ROLE, item_value[1])

                else:
                    item = QListWidgetItem(f"{indent_str}{item_value}")
                item.setFlags(
                    Qt.ItemFlag.ItemIsUserCheckable | Qt.ItemFlag.ItemIsEnabled
                )
                value = item.data(consts.LIST_ITEM_ROLE) or item.text()
                if value.strip() in self._selected:
                    item.setCheckState(Qt.CheckState.Checked)
                else:
                    item.setCheckState(Qt.CheckState.Unchecked)
                self.view.addItem(item)

    def add_group(self, group_name: str, items: list[str], indent: int = 0):
        indent_str = "    " * indent
        # Add group header
        header_item = QListWidgetItem(indent_str + group_name)
        header_item.setFlags(Qt.ItemFlag.ItemIsEnabled)
        header_item.setData(Qt.ItemDataRole.UserRole, False)  # Non-selectable
        self.view.addItem(header_item)

        # Add items with checkboxes
        self.add_items(items, indent + 1)

    def update_display_text(self):
        selected_items = []
        for i in range(self.view.count()):
            item = self.view.item(i)
            if item.checkState() == Qt.CheckState.Checked:
                selected_items.append(item.text().strip())

        text = ", ".join(selected_items)
        self.lineEdit().setText(text)

    def showPopup(self):
        # Override showPopup to update text when the dropdown is shown
        self.update_display_text()
        super().showPopup()

    def hidePopup(self):
        # Override hidePopup to update text when closing dropdown
        self.update_display_text()
        super().hidePopup()

    def same_selected(self, values: list[str]) -> bool:
        return set(values) == set(self._selected)

    def get_selected(self) -> list[str]:
        return [
            qt_utils.get_item_value(item)
            for item in self.view.findItems("", Qt.MatchFlag.MatchContains)
            if item.checkState() == Qt.CheckState.Checked
        ]

    def _selected_changed(self):
        self.selectedChanged.emit(self.get_selected())

    def get_unweight_chip(self, tag: str) -> str:
        if self.weight_re and self.weight_re.search(tag):
            return self.weight_re.search(tag).groups()[0]
        return tag

    def get_chip_list(self, tags: list[str]) -> list[str]:
        return [self.get_unweight_chip(tag) for tag in tags]

    def set_selected(self, values: list[str], emit: bool = True):
        """if self.same_selected(values):
        return"""

        values = self.get_chip_list(values)
        # Set the selected items and emit the signal
        self._selected = utils.dedupe_list(values)
        # Update the checked state of items based on the new selection
        for i in range(self.view.count()):
            item = self.view.item(i)

            item.setCheckState(
                Qt.CheckState.Checked
                if qt_utils.get_item_value(item) in values
                else Qt.CheckState.Unchecked
            )
        self.update_display_text()
        if emit:
            self._selected_changed()
